#include <LiquidCrystal.h>
#include <TimerOne.h>


//------------------------------Control de pantallas-----------------------------//
/*Variables utilizadas para el control de las distintas pantallas mostradas*/
int READY = 0;
int screen = 0;
int time = 0;

//Variables para el modo MMA
float Tmin = 650;
float screenMin = 0;
float Tmax = -150;
float screenMax = 0;
float avg = 0;
unsigned long avg_counter = 0;
float screenAvg = 0;
int push_MMA=0;

//Variables para el modo HOLD
float Tscreen = 0;
int push_HOLD = 0;
//Variables para el modo OTD
int SET_OTD = 0;
//Constante para el modo OTD (límite detección de termopar roto)
int lim_OTD = 700;

//Variables para el modo LOWBATT
int SET_LOWBATT = 0;
//Constante para el modo LOWBATT (límite detección de pila agotada)
int lim_inf_LOWBATT = 490;
int lim_sup_LOWBATT = 510;

/*Constantes utilizadas para el control de las distintas pantallas mostradas*/
int time_lim = 30;


//-------------------------apagado y autoapagado-----------------------------//
/*Variables utilizadas para el apagado y autoapagado*/
int LOCK_OFF = 1;
int t_AUTO = 0;
int AUTO = 1;

/*Constantes utilizadas para el apagado y autoapagado*/
int t_AUTO_lim = 300;//segundos

//-------------------Variables para la adquisición de datos--------------------------//
int DATA_counter = 0;
//Vectores que almacenan los diez valores adc del LM35 y termopar, respectivamente, uno cada 100ms
int a0[10] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
int a1[10] = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1};
//Variables valores medios de elementos de los vectores a0 y a1 
float adc0 = 0;
float adc1 = 0;


//-------------------Constantes setup entradas y salidas-----------------------//
/*Entradas analógicas*/
int in_termocouple = 0; //Entrada del termopar en pin A1
int OTD = 1;            //Entrada de la detección de termopar roto
int in_LM35 = 2;        //Entrada del LM35 en pin A0
int LOWBATT = 3;        //Entrada de detecciónd de batería baja

/*Entradas y salidas digitales*/
int OFF_OUT = 6;        //Salida digital señal de apagado
int OFF = 7;            //Entrada digital pulsador ON/OFF
int MMA = 8;            //Entrada digital pulsador máximo, mínimo y valor medio de la temperatura
int HOLD = 9;           //Entrada digital pulsador modo HOLD
int CLEAR = 13;         //Entrada digital pulsador CLEAR
/*Entradas y salidas digitales para el display*/
LiquidCrystal lcd( 12, 10, 11, 4, 5, 2, 3);





void setup() {
  
   lcd.begin(16,2);   //Inicialización de filas y columnas del display
   analogReference(EXTERNAL);   //Selección de la referencia para el convertidor ADC externa
   pinMode(OFF_OUT,OUTPUT);     //Configuración del pin OFF_OUT como salida digital
   pinMode(CLEAR,INPUT);        //Configuración del pin CLEAR como entrada digital 
   Timer1.initialize(100000);   //Configuración del tiempo de la interrupción a 100ms
   Timer1.attachInterrupt(acquisition);  //Configuración del nombre de la función a la que llama la interrupción
}

void loop() {
 
//------------------------------------------------------------------------------//
//------------------------------------MODO MMA----------------------------------//
//------------------------------------------------------------------------------// 
  push_MMA = 0;
  for (int i=0; i<10; i++)
  {
    if (digitalRead(MMA)==1)
    {
      push_MMA ++;
    }    
  }
  if ( push_MMA >= 6)
  {
    switch (READY)
    {
      case 0:
        screen = 1;
        screenMax=Tmax;
        screenMin=Tmin;
        screenAvg=avg/avg_counter;
        lcd.setCursor(0,1);
        lcd.print ("                ");
        lcd.setCursor(0,1);
        lcd.print("MAX");
        lcd.setCursor(9,1);
        lcd.print(screenMax,1);
        lcd.setCursor (14,1);
        lcd.write(char(223));
        lcd.setCursor (15,1);
        lcd.print ("C");
        
        do
        {
          push_MMA=0;
          for (int i=0; i<10;i++)
          {
            if(digitalRead (MMA) == 1)
            {  
                push_MMA ++;
            }
          }
        }while (push_MMA >= 6);
                                
        READY = 1;
        break;
      
      case 1:
        lcd.setCursor(0,1);
        lcd.print ("                ");
        lcd.setCursor(0,1);
        lcd.print("MIN");
        lcd.setCursor(9,1);
        lcd.print(screenMin,1);
        lcd.setCursor (14,1);
        lcd.write(char(223));
        lcd.setCursor (15,1);
        lcd.print ("C"); 
        do
        {
          push_MMA=0;
          for (int i=0; i<10;i++)
          {
            if(digitalRead (MMA) == 1)
            {  
                push_MMA ++;
            }
          }
        }while (push_MMA >= 6);
        READY = 2;
        break;
      
      case 2:
        lcd.setCursor(0,1);
        lcd.print ("                ");
        lcd.setCursor(0,1);
        lcd.print("AVG");
        lcd.setCursor( 9,1);
        lcd.print(screenAvg,1); 
        lcd.setCursor (14,1);
        lcd.write(char(223));
        lcd.setCursor (15,1);
        lcd.print ("C"); 
        do
        {
          push_MMA=0;
          for (int i=0; i<10;i++)
          {
            if(digitalRead (MMA) == 1)
            {  
                push_MMA ++;
            }
          }
        }while (push_MMA >= 6);
        READY = 3;
        break;
                                 
      case 3:
        screen = 0;
        do
        {
          push_MMA=0;
          for (int i=0; i<10;i++)
          {
            if(digitalRead (MMA) == 1)
            {  
                push_MMA ++;
            }
          }
        }while (push_MMA >= 6);
        READY = 0;
        break;        
    }
    time = 0;
    t_AUTO = 0;
  }
  
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//


//------------------------------------------------------------------------------//
//-------------------------------Modo HOLD--------------------------------------//
//------------------------------------------------------------------------------//

  push_HOLD = 0;
  for (int i=0; i<10; i++)
  {
    if (digitalRead(HOLD)==1)
    {
      push_HOLD ++;
    }    
  }

  if (push_HOLD >=6)
  {
    switch (READY)
    {
      case 0:
        screen = 1;
        if(SET_OTD ==1)
        { 
          lcd.setCursor(0,1);
          lcd.print("                ");
          lcd.setCursor(0,1);
          lcd.print("HOLD");
          lcd.setCursor(10,1);
          lcd.print ("ERROR");
        }
        else
        {
          lcd.setCursor(0,1);
          lcd.print("                ");
          lcd.setCursor(0,1);
          lcd.print("HOLD");
          lcd.setCursor(9,1);
          lcd.print (Tscreen,1);
          lcd.setCursor (14,1);
          lcd.write(char(223));
          lcd.setCursor (15,1);
          lcd.print ("C"); 
        } 
        do
        {
          push_HOLD=0;
          for (int i=0; i<10;i++)
          {
            if(digitalRead (HOLD) == 1)
            {  
                push_HOLD ++;
            }
          }
        }while (push_HOLD >= 6);
        READY = 4;  
        break;
       
      case 4:
        screen = 0;
        do
        {
          push_HOLD=0;
          for (int i=0; i<10;i++)
          {
            if(digitalRead (HOLD) == 1)
            {  
                push_HOLD ++;
            }
          }
        }while (push_HOLD >= 6);
        READY = 0;
        break;
      }
    time = 0;
    t_AUTO = 0;
  }
  
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------// 
  


//------------------------------------------------------------------------------//
//----------------------------------MODO CLEAR----------------------------------//
//------------------------------------------------------------------------------//   
  if (digitalRead (CLEAR) == 1)
  {
    switch (READY)
    {
      case 0:
        screen = 1;
        lcd.setCursor (0,1);
        lcd.print("                ");
        lcd.setCursor (0,1);
        lcd.print("RESET M M A");
        Tmax=-150;
        Tmin=650;
        avg=0;
        avg_counter=0;         
        break;
      
      case 4:
        lcd.setCursor(0,1);
        lcd.print("                ");
        lcd.setCursor(0,1);
        lcd.print("AUTO OFF");
        AUTO =0;
        break;
    }
    while (digitalRead (CLEAR) == 1)
    {
    }
    delay(1000);
    screen = 0;
    READY = 0;
    time = 0;
    t_AUTO = 0;
  }
  
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------//
//------------------------------------------------------------------------------// 

//---------Regreso a pantalla principal---------------//   
 if (time > time_lim && screen != 0)
 {
   screen = 0;
   READY = 0;
 }
//----------------------------------------------------//
 
  
//-----------------------------------------------------------------------------//
//-----------------------------APAGADO Y AUTOAPAGADO---------------------------//
//-----------------------------------------------------------------------------//

  if (t_AUTO > t_AUTO_lim && AUTO == 1)
  {
    digitalWrite(OFF_OUT,1);
  }
  else
  {   
    if (digitalRead(OFF) == 1)
    {
      if(LOCK_OFF == 0)
      {
        while(digitalRead (OFF) ==1 )
        {
        }
        delay(50);
        digitalWrite(OFF_OUT,1); 
      }
      else
      {
        while(digitalRead (OFF) == 1)
        {
        }       
        LOCK_OFF = 0;
        t_AUTO = 0;
      }  
    }
    else
    {
      LOCK_OFF = 0;
    }
  }    
  
//-------------------------------------------------------------------------//
//-------------------------------------------------------------------------//
//-------------------------------------------------------------------------//




}

